#!/usr/bin/env python
#====================================================
# GoZ Script: Saving GoZ Mesh
# This file is for GoZ file-exchange with ZBrush
#====================================================
import lx
import os
import time
import types

gGoZBrushPath = "/Users/Shared/Pixologic/GoZBrush/"
gDebug = 0



# ===============
#  Debug purpose 
# ===============
def printDebug(str):
	"Prints a string only if debug is activated"
	if (gDebug):
		lx.out(str)
	return

def safeGetItemName():
	try:
		return lx.eval('item.name ?')
	except:
		return ""


# ================
#  Reload texture
# ================
def reloadTexture(textureName, textureDisplayName):
	lx.eval('select.drop item')
	lx.eval('select.item "%s" set' %(textureName))
	#itemName = lx.eval('item.name ?')
	itemName = safeGetItemName()
	if (itemName == textureName):
		printDebug('GoZ Note: reloading %s' %(textureDisplayName))
		lx.eval("!!clip.reload")



# ================================
#  Invert normal map Green channel
# ================================
def invertNormalMapGreen(textureName):
	imageName = textureName + " (Image)"
	lx.eval('select.drop item')
	lx.eval('select.item "%s" set' %(imageName))
	itemName = safeGetItemName()
	if (itemName == imageName):
		printDebug('GoZ Note: invert green %s' %(textureName))
		lx.eval('item.channel imageMap$greenInv true')
	else:
		imageCount = lx.eval('query sceneservice imageMap.N ? all')
		for imageIndex in range(0, imageCount):
			imageName = lx.eval('query sceneservice imageMap.name ? %s' %(imageIndex))
			if (textureName in imageName):
				printDebug('GoZ Note: invert green %s' %(textureName))
				lx.eval('select.item "%s" set' %(imageName))
				lx.eval('item.channel imageMap$greenInv true')
				break



# ==========================
#  Save / Restore selection
# ==========================
# Save the selection as a tuple of selected item IDs.
# To avoid problems with 0 or 1 item selected,
# 2 extra unused elements are appended at the end of the tuple:
# do not process those 2 last elements!
# -------------------------------------------------------------
def saveSelection():
	sel = lx.eval('query sceneservice selection ? all')
	if type(sel) == tuple:
		selList = sel + ("unused1", "unused2")
	else:
		selList = (sel, "unused1", "unused2")
	return selList

# Restore a selection saved using saveSelection.
# ----------------------------------------------
def restoreSelection(selList):
	lx.eval('select.drop item')
	lcount = len(selList)-2
	for i in range(0, lcount):
		itemID = selList[i]
		lx.eval('select.item %s add' %(itemID))


def getItemName(itemID):
	result = ""
	printDebug('getItemName(%s)...' %(itemID))
	if (itemID != None):
		lx.eval('select.drop item')
		lx.eval('select.item "%s" set' %(itemID))
		#selItemName = lx.eval('item.name ?')
		selItemName = safeGetItemName()
		printDebug('  - selItemName = "%s"' %(selItemName))
		result = ""
		if (selItemName != ""):
			selItemID = lx.eval('query sceneservice item.id ? "%s"' %(selItemName))
			printDebug('  - selItemID = %s' %(selItemID))
			if (selItemID == itemID):
				result = selItemName
	return result

def getItemID(itemName):
	lx.eval('select.drop item')
	lx.eval('select.item "%s" set' %(itemName))
	#selItemName = lx.eval('item.name ?')
	selItemName = safeGetItemName()
	result = ""
	if (itemName == selItemName):
		result = lx.eval('query sceneservice item.id ? "%s"' %(itemName))
	return result

def getGoZItemID(itemName, gozID):
	itemID = getItemID(itemName)
	if (itemID != ""):
		gozAttr = lx.eval('item.tag string GOZB ?')
		if (gozAttr != gozID):
			itemID = ""
	return itemID

def getGoZMeshID(gozID):
	return getGoZItemID(gozID, gozID)

def getGoZGroupID(gozID):
	return getGoZItemID(gozID+"_GoZ", gozID)

def getGoZMatID(gozID):
	return getGoZItemID("Item: "+gozID+"_GoZ", gozID)

def getParentName(itemID):
	parentName = ""
	if (itemID != ""):
		parent = lx.eval('query sceneservice item.parent ? %s' %(itemID))
		parentName = getItemName(parent)
	return parentName

def deleteSelection():
	sel = lx.eval('query sceneservice selection ? all')
	if (type(sel) == tuple) or ((sel != "None") and (sel != None)):
		lx.eval('!!item.delete')

def deleteItem(item):
	if (item != ""):
		lx.eval('select.drop item')
		lx.eval('select.item %s add' %(item))
		deleteSelection()

def deleteItems(items):
	lx.eval('select.drop item')
	itemCount = len(items)
	for itemIndex in range(0, itemCount):
		item = items[itemIndex]
		if (item != ""):
			lx.eval('select.item %s add' %(item))
	deleteSelection()

def dialog(title, msg, type):
	try:
		lx.eval('dialog.setup %s' %(type))
		lx.eval('dialog.title "%s"' %(title))
		lx.eval('dialog.msg "%s"' %(msg))
		lx.eval('dialog.result ok')
		lx.eval('dialog.open')
		result = lx.eval('dialog.result ?')
		return result
		
	except:
		result = lx.eval('dialog.result ?')
		return result


# ========================
#  Import the GoZ project
# ========================
class GoZObject:
	m_path = ""
	m_name = ""
	m_id = ""
	m_prevMeshID = ""
	m_prevGroupID = ""
	m_prevMatID = ""
	m_parentName = ""
	m_abort = 0

	def __init__(self, gozPath):
		printDebug('Creating object with path "%s"...' %(gozPath))
		self.m_path = gozPath
		printDebug('  self.m_path="%s"' %(self.m_path))
		self.m_name = os.path.basename(self.m_path)
		printDebug('  self.m_name="%s"' %(self.m_name))
		self.m_id = ""
		printDebug('  self.m_id="%s"' %(self.m_id))
		self.m_prevMeshID = getGoZMeshID(self.m_name)
		printDebug('  self.m_prevMeshID="%s"' %(self.m_prevMeshID))
		self.m_prevGroupID = getGoZGroupID(self.m_name)
		printDebug('  self.m_prevGroupID="%s"' %(self.m_prevGroupID))
		self.m_prevMatID = getGoZMatID(self.m_name)
		printDebug('  self.m_prevMatID="%s"' %(self.m_prevMatID))
		self.m_parentName = getParentName(self.m_prevMeshID)
		printDebug('  self.m_parentName="%s"' %(self.m_parentName))
		self.m_abort = 0
		printDebug('  self.m_abort=%d' %(self.m_abort))
		printDebug('  ...object created!')

	def deletePrevIDs(self):
		printDebug('before deleting items...')
		deleteItems((self.m_prevMeshID, self.m_prevGroupID, self.m_prevMatID))
		printDebug('after deleting items...')
		self.m_prevMeshID = ""
		self.m_prevGroupID = ""
		self.m_prevMatID = ""

	def doImport(self):
		deleteItems((self.m_name+"_GoZ", "Item: "+self.m_name, "Item: "+self.m_name+"_GoZ", "GoZMesh_" + self.m_name))
		item = getItemID(self.m_name)
		itemToRename = ""
		if (item != ""):
			title = "Importing GoZ object " + self.m_name
			msg = "There is already an existing item [" + self.m_name + "] in the Modo scene.\n"
			msg = msg + "Do you want to replace it?\n'Yes' to replace, 'No' to rename the existing mesh " + self.m_name + ", 'Cancel' to cancel import"
			res = dialog("GoZ note", msg, "yesNoCancel")
			printDebug('Import duplicated object "%s" dialog result is "%s"' %(self.m_name, res))
			if (res == "ok"):
				deleteItem(item)
			if (res == "no"):
				itemToRename = "GoZTemp_" + self.m_name
				lx.eval('item.name "%s"' %(itemToRename))
			if (res == "cancel"):
				self.m_abort = 1

		if (self.m_abort == 0):
			printDebug('Loading scene "%s"...' %(self.m_path))
			lx.eval('select.drop item')
			lx.eval('!!scene.open "%s.GoZ" Import' %(self.m_path))
			self.m_id = lx.eval('query sceneservice item.children ? "%s"' %(self.m_name))
			if type(self.m_id) == tuple:
				self.m_id = self.m_id[0]
			#lx.eval('select.drop item')
			#lx.eval('select.item %s set' %(self.m_name))
			#lx.eval('select.itemHierarchy "%s" root set' %(self.m_name))
			#lx.eval('select.itemHierarchy "%s" down set' %(self.m_name))
			#itemName = lx.eval('item.name ?')
			#self.m_id = getItemID(itemName)
			if (self.m_id == ""):
				printDebug('!!!Imported GoZMesh %s => cannot find meshID' %(self.m_name))
				return
			if (self.m_id == "None"):
				printDebug('!!!Imported GoZMesh %s => cannot find meshID' %(self.m_name))
				return
			reloadTexture(self.m_name + "_TXTR", "Texture")
			reloadTexture(self.m_name + "_DM", "DispMap")
			reloadTexture(self.m_name + "_NM", "NormalMap")
			invertNormalMapGreen(self.m_name + "_NM")
			printDebug('  ... scene loaded!')
			lx.eval('select.drop item')
			lx.eval('select.item %s set' %(self.m_name))
			lx.eval('item.name "%s_GoZ"' %(self.m_name))
			lx.eval('select.drop item')
			printDebug('  Select itemID "%s"' %(self.m_id))
			lx.eval('select.item %s set' %(self.m_id))
			printDebug('  rename in "%s"' %(self.m_name))
			lx.eval('item.name "%s"' %(self.m_name))
			printDebug('  tag as "%s"' %(self.m_name))
			lx.eval('item.tag string GOZB "%s"' %(self.m_name));
			if (itemToRename != ""):
				printDebug('Renaming item "%s"...' %(itemToRename))
				lx.eval('select.item "%s" set' %(itemToRename))
				lx.eval('item.name "%s"' %(self.m_name))
				printDebug('  Rename done!')


	def reparent(self):
		printDebug('Reparent of GoZObject(%s) with abort=%d to parent named "%s"' %(self.m_name, self.m_abort, self.m_parentName))
		if (self.m_abort == 0) and (self.m_parentName != ""):
			itemName = getItemName(self.m_id)
			parentID = getItemID(self.m_parentName)
			printDebug('  prepare to reparent item(ID=%s, name="%s") to parent item(ID=%s, name="%s")...' %(self.m_id, itemName, parentID, self.m_parentName))
			#lx.eval('select.subItem %s set' %(self.m_id))
			lx.eval('item.parent %s %s' %(self.m_id, parentID))
			printDebug('  ...reparent done')

try:
	# For MacOSX only
	fileName = "/Users/Shared/Pixologic/GoZBrush/GoZ_AppWait.txt";
	file = open(fileName, "r")
	file.close()
	os.remove(fileName)

	lx.out('Importing GoZ...')

	# Reads the mesh list to import.
	# ------------------------------
	fileName = gGoZBrushPath + "GoZ_ObjectList.txt"
	printDebug('Opening file "%s"' %(fileName))
	file = open(fileName, "r")
	printDebug('File open"%s"' %(fileName))
	tempPathList = []
	tempParentList = []
	tempOriginalNameList = []
	tempList = []
	path = file.readline()
	while (path != ""):
		path = path.splitlines()[0]
		if (path != ""):
			object = GoZObject(path)
			tempList.append(object)
		path = file.readline()
	file.close()

	objects = tuple(tempList)
	count = len(objects)
	for i in range(0, count):
		objects[i].deletePrevIDs()
	for i in range(0, count):
		objects[i].doImport()
	for i in range(0, count):
		objects[i].reparent()

except:
	printDebug('Script called twice or error in script')
